@@ -8,12 +8,12 @@
 #include <spa/support/log.h>
 
 #undef SPA_LOG_TOPIC_DEFAULT
-#define SPA_LOG_TOPIC_DEFAULT avb_log_topic
-extern struct spa_log_topic *avb_log_topic;
+#define SPA_LOG_TOPIC_DEFAULT &avb_log_topic
+extern struct spa_log_topic avb_log_topic;
 
 static inline void avb_log_topic_init(struct spa_log *log)
 {
-	spa_log_topic_init(log, avb_log_topic);
+	spa_log_topic_init(log, &avb_log_topic);
 }
 
 #endif /* SPA_AVB_H */
