@@ -74,7 +74,7 @@ static inline size_t codec_get_caps_size(const struct media_codec *codec)
 }
 
 static int codec_fill_caps(const struct media_codec *codec, uint32_t flags,
-		uint8_t caps[A2DP_MAX_CAPS_SIZE])
+		const struct spa_dict *settings, uint8_t caps[A2DP_MAX_CAPS_SIZE])
 {
 	size_t actual_conf_size = codec_get_caps_size(codec);
 	const a2dp_aptx_t a2dp_aptx = {
@@ -300,8 +300,17 @@ static void *codec_init(const struct media_codec *codec, uint32_t flags,
 		void *config, size_t config_len, const struct spa_audio_info *info,
 		void *props, size_t mtu)
 {
-	struct impl *this;
+	struct impl *this = NULL;
+	a2dp_aptx_t conf;
 	int res;
+	int frequency;
+
+	if (config_len < sizeof(conf)) {
+		res = -EINVAL;
+		goto error;
+	}
+
+	memcpy(&conf, config, sizeof(conf));
 
 	if ((this = calloc(1, sizeof(struct impl))) == NULL)
 		goto error_errno;
@@ -322,10 +331,17 @@ static void *codec_init(const struct media_codec *codec, uint32_t flags,
 	this->frame_length = this->hd ? 6 : 4;
 	this->codesize = 4 * 3 * 2;
 
+	frequency = media_codec_get_config(aptx_frequencies, SPA_N_ELEMENTS(aptx_frequencies), conf.frequency);
+	if (frequency < 0) {
+		res = -EINVAL;
+		goto error;
+	}
+
 	if (this->hd)
 		this->max_frames = (this->mtu - sizeof(struct rtp_header)) / this->frame_length;
 	else if (codec_is_ll(codec))
-		this->max_frames = SPA_MIN(256u, this->mtu) / this->frame_length;
+		/* try to make 7.5ms packets */
+		this->max_frames = SPA_MIN((unsigned)frequency * 75u/10000u / 4u, this->mtu / this->frame_length);
 	else
 		this->max_frames = this->mtu / this->frame_length;
 
@@ -433,6 +449,14 @@ static int codec_decode(void *data,
 	return res;
 }
 
+static void codec_get_delay(void *data, uint32_t *encoder, uint32_t *decoder)
+{
+	if (encoder)
+		*encoder = 90;
+	if (decoder)
+		*decoder = 0;
+}
+
 /*
  * mSBC duplex codec
  *
@@ -611,6 +635,7 @@ const struct media_codec a2dp_codec_aptx = {
 	.decode = codec_decode,
 	.reduce_bitpool = codec_reduce_bitpool,
 	.increase_bitpool = codec_increase_bitpool,
+	.get_delay = codec_get_delay,
 };
 
 
@@ -634,6 +659,7 @@ const struct media_codec a2dp_codec_aptx_hd = {
 	.decode = codec_decode,
 	.reduce_bitpool = codec_reduce_bitpool,
 	.increase_bitpool = codec_increase_bitpool,
+	.get_delay = codec_get_delay,
 };
 
 #define APTX_LL_COMMON_DEFS				\
@@ -649,7 +675,8 @@ const struct media_codec a2dp_codec_aptx_hd = {
 	.start_encode = codec_start_encode,		\
 	.encode = codec_encode,				\
 	.reduce_bitpool = codec_reduce_bitpool,		\
-	.increase_bitpool = codec_increase_bitpool
+	.increase_bitpool = codec_increase_bitpool,	\
+	.get_delay = codec_get_delay
 
 
 const struct media_codec a2dp_codec_aptx_ll_0 = {
