@@ -8,6 +8,8 @@
 
 #include <stddef.h>
 
+#include <sstream>
+
 #include <spa/support/plugin.h>
 #include <spa/support/log.h>
 #include <spa/support/loop.h>
@@ -53,32 +55,25 @@ struct impl {
 
 }
 
-static const libcamera::Span<const int64_t> cameraDevice(
-			const Camera *camera)
+static const libcamera::Span<const int64_t> cameraDevice(const Camera& camera)
 {
-	const ControlList &props = camera->properties();
-
-	if (auto devices = props.get(properties::SystemDevices))
+	if (auto devices = camera.properties().get(properties::SystemDevices))
 		return devices.value();
 
 	return {};
 }
 
-static std::string cameraModel(const Camera *camera)
+static std::string cameraModel(const Camera& camera)
 {
-	const ControlList &props = camera->properties();
-
-	if (auto model = props.get(properties::Model))
+	if (auto model = camera.properties().get(properties::Model))
 		return std::move(model.value());
 
-	return camera->id();
+	return camera.id();
 }
 
-static const char *cameraLoc(const Camera *camera)
+static const char *cameraLoc(const Camera& camera)
 {
-	const ControlList &props = camera->properties();
-
-	if (auto location = props.get(properties::Location)) {
+	if (auto location = camera.properties().get(properties::Location)) {
 		switch (location.value()) {
 		case properties::CameraLocationFront:
 			return "front";
@@ -92,11 +87,9 @@ static const char *cameraLoc(const Camera *camera)
 	return nullptr;
 }
 
-static const char *cameraRot(const Camera *camera)
+static const char *cameraRot(const Camera& camera)
 {
-	const ControlList &props = camera->properties();
-
-	if (auto rotation = props.get(properties::Rotation)) {
+	if (auto rotation = camera.properties().get(properties::Rotation)) {
 		switch (rotation.value()) {
 		case 90:
 			return "90";
@@ -119,44 +112,48 @@ static int emit_info(struct impl *impl, bool full)
 	uint32_t n_items = 0;
 	struct spa_device_info info;
 	struct spa_param_info params[2];
-	char path[256], name[256], devices_str[128];
-	struct spa_strbuf buf;
+	Camera& camera = *impl->camera;
 
 	info = SPA_DEVICE_INFO_INIT();
 
 	info.change_mask = SPA_DEVICE_CHANGE_MASK_PROPS;
 
 #define ADD_ITEM(key, value) items[n_items++] = SPA_DICT_ITEM_INIT(key, value)
-	snprintf(path, sizeof(path), "libcamera:%s", impl->device_id.c_str());
-	ADD_ITEM(SPA_KEY_OBJECT_PATH, path);
+
+	const auto path = "libcamera:" + impl->device_id;
+	ADD_ITEM(SPA_KEY_OBJECT_PATH, path.c_str());
+
 	ADD_ITEM(SPA_KEY_DEVICE_API, "libcamera");
 	ADD_ITEM(SPA_KEY_MEDIA_CLASS, "Video/Device");
 	ADD_ITEM(SPA_KEY_API_LIBCAMERA_PATH, impl->device_id.c_str());
 
-	if (auto location = cameraLoc(impl->camera.get()))
+	if (auto location = cameraLoc(camera))
 		ADD_ITEM(SPA_KEY_API_LIBCAMERA_LOCATION, location);
-	if (auto rotation = cameraRot(impl->camera.get()))
+	if (auto rotation = cameraRot(camera))
 		ADD_ITEM(SPA_KEY_API_LIBCAMERA_ROTATION, rotation);
 
-	const auto model = cameraModel(impl->camera.get());
+	const auto model = cameraModel(camera);
 	ADD_ITEM(SPA_KEY_DEVICE_PRODUCT_NAME, model.c_str());
 	ADD_ITEM(SPA_KEY_DEVICE_DESCRIPTION, model.c_str());
 
-	snprintf(name, sizeof(name), "libcamera_device.%s", impl->device_id.c_str());
-	ADD_ITEM(SPA_KEY_DEVICE_NAME, name);
+	const auto name = "libcamera_device." + impl->device_id;
+	ADD_ITEM(SPA_KEY_DEVICE_NAME, name.c_str());
 
-	auto device_numbers = cameraDevice(impl->camera.get());
+	auto device_numbers = cameraDevice(camera);
+	std::string devids;
 
 	if (!device_numbers.empty()) {
-		spa_strbuf_init(&buf, devices_str, sizeof(devices_str));
+		std::ostringstream s;
+
 
 		/* encode device numbers into a json array */
-		spa_strbuf_append(&buf, "[ ");
-		for(int64_t device_number : device_numbers)
-			spa_strbuf_append(&buf, "%" PRId64 " ", device_number);
+		s << "[ ";
+		for (const auto& devid : device_numbers)
+			s << devid << ' ';
+		s << ']';
 
-		spa_strbuf_append(&buf, "]");
-		ADD_ITEM(SPA_KEY_DEVICE_DEVIDS, devices_str);
+		devids = std::move(s).str();
+		ADD_ITEM(SPA_KEY_DEVICE_DEVIDS, devids.c_str());
 	}
 
 #undef ADD_ITEM
