@@ -12,6 +12,9 @@
 #include <spa/pod/parser.h>
 #include <spa/debug/types.h>
 
+#define PW_API_CORE_IMPL	SPA_EXPORT
+#define PW_API_REGISTRY_IMPL	SPA_EXPORT
+
 #include "pipewire/pipewire.h"
 #include "pipewire/private.h"
 
@@ -20,6 +23,18 @@
 PW_LOG_TOPIC_EXTERN(log_core);
 #define PW_LOG_TOPIC_DEFAULT log_core
 
+static void core_event_info(void *data, const struct pw_core_info *info)
+{
+	struct pw_core *this = data;
+	if (info && info->props) {
+		static const char * const keys[] = {
+			"default.clock.quantum-limit",
+			NULL
+		};
+		pw_properties_update_keys(this->context->properties, info->props, keys);
+	}
+}
+
 static void core_event_ping(void *data, uint32_t id, int seq)
 {
 	struct pw_core *this = data;
@@ -111,6 +126,7 @@ static void core_event_remove_mem(void *data, uint32_t id)
 
 static const struct pw_core_events core_events = {
 	PW_VERSION_CORE_EVENTS,
+	.info = core_event_info,
 	.error = core_event_error,
 	.ping = core_event_ping,
 	.done = core_event_done,
@@ -476,6 +492,15 @@ struct pw_mempool * pw_core_get_mempool(struct pw_core *core)
 	return core->pool;
 }
 
+SPA_EXPORT
+void pw_core_add_proxy_listener(struct pw_core *object,
+			struct spa_hook *listener,
+			const struct pw_proxy_events *events,
+			void *data)
+{
+	pw_proxy_add_listener((struct pw_proxy *)object, listener, events, data);
+}
+
 SPA_EXPORT
 int pw_core_disconnect(struct pw_core *core)
 {
